import datetime
import functools
import json
import operator
from collections import Counter

import redis

from pyspark import SparkConf
from pyspark.sql import SparkSession
from tool import (get_jerry_test, get_tag3_user_log, send_email, write_user_portrait,get_user_portrait_log,user_portrait_action_statistic,write_user_portrait_action_divided)


# [{'激光': 1.949194898204873}, {'手术': 1.949194898204873}, {'手术': 1.949194898204873}, {'手术': 1.949194898204873}]
# {'手术': 5.8475846946146195, '激光': 1.949194898204873}
def merge_values(list_of_dict):
    d = dict(functools.reduce(operator.add, map(Counter, list_of_dict)))
    return dict(sorted(d.items(), key=lambda x: x[1], reverse=True))


# [("a", 1), ("b", 2)]
# {'a': 1, 'b': 2}
# [{'缩阴道激光': 1.949194898204873}, {'': 1.949194898204873}, {'': 1.949194898204873}, {'私处手术': 1.949194898204873}]
def make_dict_from_pair(x):
    if "" in x[0]:
        x[0].remove("")
    return dict(zip(x[0], [x[1]] * len(x[0])))


def update_tag3_user_portrait(cl_id):
    user_df = get_tag3_user_log(cl_id)
    # print('*'*100)
    # print('tag3_user_log cl_id',cl_id)
    # print(user_df)
    # print('-'*100)
    if not user_df.empty:
        user_df["first_solutions"] = list(zip(user_df["first_solutions"].apply(lambda x: x.split(",")), user_df["tag_score"]))
        user_df["second_solutions"] = list(zip(user_df["second_solutions"].apply(lambda x: x.split(",")), user_df["tag_score"]))
        user_df["first_demands"] = list(zip(user_df["first_demands"].apply(lambda x: x.split(",")), user_df["tag_score"]))
        user_df["second_demands"] = list(zip(user_df["second_demands"].apply(lambda x: x.split(",")), user_df["tag_score"]))
        user_df["first_positions"] = list(zip(user_df["first_positions"].apply(lambda x: x.split(",")), user_df["tag_score"]))
        user_df["second_positions"] = list(zip(user_df["second_positions"].apply(lambda x: x.split(",")), user_df["tag_score"]))
        user_df["projects"] = list(zip(user_df["projects"].apply(lambda x: x.split(",")), user_df["tag_score"]))

        user_df["first_solutions_dict"] = user_df["first_solutions"].apply(lambda x: make_dict_from_pair(x))
        user_df["second_solutions_dict"] = user_df["second_solutions"].apply(lambda x: make_dict_from_pair(x))
        user_df["first_demands_dict"] = user_df["first_demands"].apply(lambda x: make_dict_from_pair(x))
        user_df["second_demands_dict"] = user_df["second_demands"].apply(lambda x: make_dict_from_pair(x))
        user_df["first_positions_dict"] = user_df["first_positions"].apply(lambda x: make_dict_from_pair(x))
        user_df["second_positions_dict"] = user_df["second_positions"].apply(lambda x: make_dict_from_pair(x))
        user_df["projects_dict"] = user_df["projects"].apply(lambda x: make_dict_from_pair(x))

        first_solutions_list = user_df["first_solutions_dict"].tolist()
        second_solutions_list = user_df["second_solutions_dict"].tolist()
        first_demands_list = user_df["first_demands_dict"].tolist()
        second_demands_list = user_df["second_demands_dict"].tolist()
        first_positions_list = user_df["first_positions_dict"].tolist()
        second_positions_list = user_df["second_positions_dict"].tolist()
        projects_list = user_df["projects_dict"].tolist()

        first_demands_score = merge_values(first_demands_list)
        second_demands_score = merge_values(second_demands_list)
        first_solutions_score = merge_values(first_solutions_list)
        second_solutions_score = merge_values(second_solutions_list)
        first_positions_score = merge_values(first_positions_list)
        second_positions_score = merge_values(second_positions_list)
        projects_score = merge_values(projects_list)

        res = {
            "first_demands": first_demands_score,
            "second_demands": second_demands_score,
            "first_solutions": first_solutions_score,
            "second_solutions": second_solutions_score,
            "first_positions": first_positions_score,
            "second_positions": second_positions_score,
            "projects": projects_score
        }

        key = "doris:user_portrait:tag3:device_id:" + str(cl_id)
        redis_client = redis.StrictRedis.from_url("redis://:ReDis!GmTx*0aN9@172.16.40.173:6379")

        if (len(first_demands_score.keys()) > 0) or (len(second_demands_score.keys()) > 0) or \
           (len(first_solutions_score.keys()) > 0) or (len(second_solutions_score.keys()) > 0) or \
           (len(first_positions_score.keys()) > 0) or (len(second_positions_score.keys()) > 0) or \
           (len(projects_score.keys()) > 0):
            redis_client.set(key, json.dumps(res))
            redis_client.expire(key, 60 * 60 * 24 * 30)

            # only need the first time
            # key2 = "doris:user_portrait:tag3:increment_update:device_id:" + str(cl_id)
            # res2 = {
            #     "first_demands": list(first_demands_score.keys()),
            #     "second_demands": list(second_demands_score.keys()),
            #     "first_solutions": list(first_solutions_score.keys()),
            #     "second_solutions": list(second_solutions_score.keys()),
            #     "first_positions": list(first_positions_score.keys()),
            #     "second_positions": list(second_positions_score.keys()),
            #     "projects": list(projects_score.keys())
            # }
            # redis_client.set(key2, json.dumps(res2))
            # redis_client.expire(key2, 60 * 60 * 24 * 30)
            # print('*'*10)
            # write_user_portrait(cl_id, ",".join(first_solutions_score.keys()), ",".join(second_solutions_score.keys()),
            #                     ",".join(first_demands_score.keys()), ",".join(second_demands_score.keys()),
            #                     ",".join(first_positions_score.keys()), ",".join(second_positions_score.keys()),
            #                     ",".join(projects_score.keys()))

        first_solutions = []
        first_positions = []
        first_demands = []
        second_demands = []
        second_positions = []
        second_solutions = []
        projects = []

        # cl_ids = [1, 3]
        # print('*'*100)
        # print('INPUT cl_ids:',cl_id)
        # print('OUTPUT data:')
        data = user_portrait_action_statistic(cl_id)
        print('=' * 100)
        # print(data)
        print('cl_id',cl_id)
        print('='*100)
        for key, values in data.items():
            cl_id_single = key
            for item in values.keys():
                if values[item]:

                    first_solutions = values[item].get("first_solutions", [])
                    first_positions = values[item].get("first_positions", [])
                    first_demands = values[item].get("first_demands", [])
                    second_demands = values[item].get("second_demands", [])
                    second_positions = values[item].get("second_positions", [])
                    second_solutions = values[item].get("second_solutions", [])
                    projects = values[item].get("projects", [])
                    score = values[item].get("score",0)

                    first_solutions = [i for i in first_solutions if i != '']
                    second_solutions = [i for i in second_solutions if i != '']
                    first_demands = [i for i in first_demands if i != '']
                    second_demands = [i for i in second_demands if i != '']
                    first_positions = [i for i in first_positions if i != '']
                    second_positions = [i for i in second_positions if i != '']
                    projects = [i for i in projects if i != '']


                    write_user_portrait_action_divided(cl_id=cl_id_single, event_cn=item, score = score,first_solutions=set(first_solutions),
                                                       first_positions=set(first_positions),
                                                       first_demands=set(first_demands),
                                                       second_demands=set(second_demands),
                                                       second_positions=set(second_positions),
                                                       second_solutions=set(second_solutions),
                                                       projects=set(projects),
                                                       )

        return cl_id


def consume_kafka():
    sql = "select distinct cl_id from kafka_tag3_log where log_time > UNIX_TIMESTAMP(DATE_SUB(NOW(), INTERVAL 30 day))"
    print('*'*100)
    print('consume kafka before jerry')
    db, cursor = get_jerry_test()
    print('connection stauts 1:', db.open)
    db.close()
    db.ping(reconnect=True)
    print('connection stauts 2:', db.open)
    cursor.execute(sql)
    device_ids_lst = [i[0] for i in cursor.fetchall()]
    db.close()
    cursor.close()

    # device_ids_lst = ["androidid_a25a1129c0b38f7b"]

    sparkConf = SparkConf().set("spark.hive.mapred.supports.subdirectories", "true") \
                           .set("spark.hadoop.mapreduce.input.fileinputformat.input.dir.recursive", "true") \
                           .set("spark.tispark.plan.allow_index_double_read", "false") \
                           .set("spark.tispark.plan.allow_index_read", "true") \
                           .set("spark.sql.extensions", "org.apache.spark.sql.TiExtensions") \
                           .set("spark.tispark.pd.addresses", "172.16.40.170:2379") \
                           .set("spark.io.compression.codec", "lzf") \
                           .set("spark.driver.maxResultSize", "8g") \
                           .set("spark.sql.avro.compression.codec", "snappy")

    # try:
    spark = SparkSession.builder.config(conf=sparkConf).enableHiveSupport().getOrCreate()
    spark.sparkContext.setLogLevel("WARN")
    spark.sparkContext.addPyFile("/srv/apps/ffm-baseline/eda/smart_rank/tool.py")

    device_ids_lst_rdd = spark.sparkContext.parallelize(device_ids_lst, numSlices=1000)
    result = device_ids_lst_rdd.repartition(100).map(lambda x: update_tag3_user_portrait(x))
    # result.foreach(print)
    result.collect()
    spark.stop()
    # except Exception as e:
    #     send_email("tag3_update_user_portrait_offline", "tag3_update_user_portrait_offline", e)


if __name__ == "__main__":
    start = datetime.datetime.now()
    consume_kafka()
    end = datetime.datetime.now()
    print(end - start)
    print("done")
